%
%  Subject : Discretization techniques using narrow-base trial functions 
%
%  Example Ej_4_4
%
%           approximation in 1D : heat equation as a system of two 1st
%           order ODEs
%           example 3.3 , page 117, Zienkiewicz & Morgan
%           using piecewise constant for q and linear for phi
%

% Number of elements to be used
Nx = 4;

% boundary conditions
phi_lef = 0;  % phi(x=0)
q_rig   = 0;  % q(x=1)

nen_phi = 2;  % piecewise linear for phi
nen_q   = 1;  % piecewise constant for q

% initial values
phi_0 = zeros(Nx+1,1);
q_0   = zeros(Nx,1);

% grid points and mesh
xnod  = (0:2*Nx)'/(2*Nx);
icone = [(1:2:2*Nx-1)',(2:2:2*Nx)',(3:2:2*Nx+1)'];

[numel,nen]=size(icone);

Ke = zeros(numel,nen_phi+nen_q,nen_phi+nen_q);
fe = zeros(numel,nen_phi+nen_q);

filename = 'ffun_Ej_4_3_Q';
kappa = 1;

% loop over each element
for k=1:numel
    psi = (0:20)'/20; 
    node1 = icone(k,1);
    node2 = icone(k,3);
    xx  = psi*(xnod(node2,1)-xnod(node1,1))+xnod(node1,1);            
    eval(['Qx=' filename '(xx);' ]);
    [N,L_N] = shape_function(xx);        

    Ke(k,1,2) = Ke(k,1,2) + trapz(xx,L_N(:,1));  
    Ke(k,3,2) = Ke(k,3,2) + trapz(xx,L_N(:,2));  
    Ke(k,2,1) = Ke(k,2,1) + kappa*trapz(xx,L_N(:,1));      
    Ke(k,2,3) = Ke(k,2,3) + kappa*trapz(xx,L_N(:,2));          
    Ke(k,2,2) = Ke(k,2,2) + 1*trapz(xx,ones(size(xx)));              
    
    fe(k,1) = fe(k,1) - trapz(xx,Qx.*N(:,1));      
    fe(k,3) = fe(k,3) - trapz(xx,Qx.*N(:,2));          
       
end

% gather Ke and fe in Kg and fg
Kg = zeros((Nx+1)+Nx,(Nx+1)+Nx);
fg = zeros((Nx+1)+Nx,1);
for k=1:numel
    node_i = icone(k,1);
    node_j = icone(k,3);
    node_e = icone(k,2); 
    Kg(node_i,node_e)=Kg(node_i,node_e) + Ke(k,1,2); 
    Kg(node_j,node_e)=Kg(node_j,node_e) + Ke(k,3,2);
    Kg(node_e,node_i)=Kg(node_e,node_i) + Ke(k,2,1);
    Kg(node_e,node_j)=Kg(node_e,node_j) + Ke(k,2,3); 
    Kg(node_e,node_e)=Kg(node_e,node_e) + Ke(k,2,2); 
    
    fg(node_i,1) = fg(node_i,1) + fe(k,1);
    fg(node_e,1) = fg(node_e,1) + fe(k,2);
    fg(node_j,1) = fg(node_j,1) + fe(k,3);        
end

% reducing the global system for fixations
free = (1:2*Nx+1)';
free(1) = [];
fg(free,1)    = fg(free,1) - Kg(free,1)    * phi_lef ;
Kg = Kg(free,free);
fg = fg(free,1);
% solver
phi_new = Kg\fg;

phi = [phi_lef;phi_new];

% Visualization
figure(1);clf;
x = xnod(1:2:end,1);
xe = 0.5*(x(1:end-1)+x(2:end));
plot(x,zeros(size(x)),'bo',x,phi(1:2:end),'k-*',xe,-phi(2:2:end-1),'r*')
xlabel('X');ylabel('(-q,phi)');title('1D heat equation with piecewise linear for phi and piecewise constant for q  ')

Xh=[]; Yh=[];
for k=1:Nx
    Xh = [ Xh , [ x(k) , x(k+1) ] ];
    Yh = [ Yh , [ -phi((k-1)*2+2) , -phi((k-1)*2+2) ] ];
end
figure(1);hold on;line(Xh,Yh);
axis([-0.05 1.05 -0.05 0.4])
grid on